﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.ComponentModel;
using Microsoft.Crm.Workflow;
using System.Workflow.ComponentModel.Compiler;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Query;
using Microsoft.Crm.SdkTypeProxy;

namespace CrmContrib.Workflow.Activities.Marketing
{
    [CrmWorkflowActivity("Add Account to Marketing List", "CrmContrib \\ Marketing")]
    public class AddAccountToMarketingListActivity : Activity
    {
        [CrmInput("Account")]
        [CrmReferenceTarget("account")]
        [ValidationOption(ValidationOption.Required)]
        public Lookup Account
        {
            get { return (Lookup)base.GetValue(AddAccountToMarketingListActivity.AccountProperty); }
            set { base.SetValue(AddAccountToMarketingListActivity.AccountProperty, value); }
        }

        public static readonly DependencyProperty AccountProperty = DependencyProperty.Register("Account", typeof(Lookup), typeof(AddAccountToMarketingListActivity));

        [CrmInput("Marketing List")]
        [CrmReferenceTarget("list")]
        [ValidationOption(ValidationOption.Required)]
        public Lookup MarketingList
        {
            get { return (Lookup)GetValue(MarketingListProperty); }
            set { SetValue(MarketingListProperty, value); }
        }

        public static readonly DependencyProperty MarketingListProperty = DependencyProperty.Register("MarketingList", typeof(Lookup), typeof(AddAccountToMarketingListActivity));

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            var contextService = executionContext.GetService<IContextService>();
            var workflowContext = contextService.Context;
            var crmService = workflowContext.CreateCrmService();

            var addRequest = new AddMemberListRequest();
            addRequest.EntityId = this.Account.Value;
            addRequest.ListId = this.MarketingList.Value;

            var addResponse = (AddMemberListResponse)crmService.Execute(addRequest);

            return base.Execute(executionContext);
        }
    }
}
