﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.ComponentModel;
using Microsoft.Crm.Workflow;
using System.Workflow.ComponentModel.Compiler;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Query;
using Microsoft.Crm.SdkTypeProxy;

namespace CrmContrib.Workflow.Activities.Marketing
{
    [CrmWorkflowActivity("Remove Lead from Marketing List", "CrmContrib \\ Marketing")]
    public class RemoveLeadFromMarketingListActivity : Activity
    {
        [CrmInput("Lead")]
        [CrmReferenceTarget("lead")]
        [ValidationOption(ValidationOption.Required)]
        public Lookup Lead
        {
            get { return (Lookup)base.GetValue(RemoveLeadFromMarketingListActivity.LeadProperty); }
            set { base.SetValue(RemoveLeadFromMarketingListActivity.LeadProperty, value); }
        }

        public static readonly DependencyProperty LeadProperty = DependencyProperty.Register("Lead", typeof(Lookup), typeof(RemoveLeadFromMarketingListActivity));

        [CrmInput("Marketing List")]
        [CrmReferenceTarget("list")]
        [ValidationOption(ValidationOption.Required)]
        public Lookup MarketingList
        {
            get { return (Lookup)GetValue(MarketingListProperty); }
            set { SetValue(MarketingListProperty, value); }
        }

        public static readonly DependencyProperty MarketingListProperty = DependencyProperty.Register("MarketingList", typeof(Lookup), typeof(RemoveLeadFromMarketingListActivity));

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            var contextService = executionContext.GetService<IContextService>();
            var workflowContext = contextService.Context;
            var crmService = workflowContext.CreateCrmService();

            var removeRequest = new RemoveMemberListRequest();
            removeRequest.EntityId = this.Lead.Value;
            removeRequest.ListId = this.MarketingList.Value;

            var addResponse = (RemoveMemberListResponse)crmService.Execute(removeRequest);

            return base.Execute(executionContext);
        }
    }
}
